<h1>IRC meetings</h1>

{% for meeting in meetings %}
<h3>{{ meeting.project }}</h3>

<ul>
{% for schedule in meeting.schedules %}
<li>{{ schedule.recurrence }} on {{ schedule.day }} at
<a href="http://www.timeanddate.com/worldclock/fixedtime.html?hour={{ schedule.utc[:2] }}&min={{ schedule.utc[2:] }}&sec=0">{{ schedule.utc }} UTC</a>
in #{{ schedule.irc }}{% if schedule.chair != meeting.chair %}, chair: {{ schedule.chair }}{% endif %}</li>
{% endfor %}
</ul>

{% if meeting.chair %}
Chair (to contact for more information): {{ meeting.chair }}</p>
{% endif %}

{{ meeting.description|urlize }}

{% endfor %}

<p><i>Page generated on {{ timestamp }} UTC</i></p>
